# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL license version 2 under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL license version 2 as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license version 2 and that you accept its terms.

'''
L{QLineEditModificationTimer} and L{TimeredQLineEdit} classes associate a
L{QTimer<qt.QTimer>} to a L{QLineEdit<qt.QLineEdit>} in order to signal user
modification only after an inactivity period.

@author: Yann Cointepas
@organization: U{NeuroSpin<http://www.neurospin.org>} and U{IFR 49<http://www.ifr49.org>}
@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''
__docformat__ = "epytext en"

from soma.qt_gui.qt_backend import QtCore, QtGui

#-------------------------------------------------------------------------------
class QLineEditModificationTimer( QtCore.QObject ):
  '''
  A L{QLineEditModificationTimer} instance is accociated to a 
  L{QLineEdit<QtGui.QLineEdit>} instance, it listen all user modification (Qt 
  signal C{'textChanged( const QString & )'}) and emit a 
  C{SIGNAL('userModification' )} when C{timerInterval} milliseconds passed
  since the last user modification.
  '''
  # Default timer interval in milliseconds
  defaultTimerInterval = 2000

  def __init__( self, qLineEdit, timerInterval=None ):
    '''
    @param qLineEdit: widget associated with this L{QLineEditModificationTimer}.
    @type  qLineEdit: L{QLineEdit<QtGui.QLineEdit>} instance
    @param timerInterval: minimum inactivity period before emitting
      C{userModification} signal. Default value is
      L{QLineEditModificationTimer.defaultTimerInterval}
    @type  timerInterval: milliseconds
    
    @see: L{TimeredQLineEdit}
    '''
    QtCore.QObject.__init__( self )
    # QLineEdit<qt.QLineEdit> instance associated with this
    # QLineEditModificationTimer
    self.qLineEdit = qLineEdit
    if timerInterval is None:
      self.timerInterval = self.defaultTimerInterval
    else:
      # minimum inactivity period before emitting C{userModification} signal.
      self.timerInterval = timerInterval
    self.__timer = QtCore.QTimer( self )
    self.__timer.setSingleShot(True)
    self.__internalModification = False
    self.connect( self.qLineEdit, QtCore.SIGNAL( 'textChanged( const QString & )'), 
                  self._userModification )
    self.connect( self.qLineEdit, QtCore.SIGNAL( 'lostFocus()' ), 
                  self._noMoreUserModification,  )
    self.connect( self.__timer, QtCore.SIGNAL( 'timeout()' ), self.modificationTimeout )

  
  def close( self ):
    self.stop()
    self.disconnect( self.qLineEdit, QtCore.SIGNAL( 'textChanged( const QString & )'), 
                  self._userModification )
    self.disconnect( self.qLineEdit, QtCore.SIGNAL( 'lostFocus()' ), 
                  self._noMoreUserModification,  )
    self.disconnect( self.__timer, QtCore.SIGNAL( 'timeout()' ), self.modificationTimeout )
    
    
  def _userModification( self ):
    if not self.__internalModification:
      self.__timer.start( self.timerInterval )

  def modificationTimeout(self):
    self.emit( QtCore.SIGNAL( 'userModification'),self.qLineEdit.text() ) 
    
  def _noMoreUserModification( self ):
    if self.__timer.isActive():
      self.__timer.stop()
      self.emit( QtCore.SIGNAL( 'userModification'),self.qLineEdit.text() )


  def stopInternalModification( self ):
    '''
    Stop emitting C{userModification} signal when associated
    L{QLineEdit<qt.QLineEdit>} is modified.
    
    @see: L{startInternalModification}
    '''
    self.__internalModification = False


  def startInternalModification( self ):
    '''
    Restart emitting C{userModification} signal when associated
    L{QLineEdit<qt.QLineEdit>} is modified.
    
    @see: L{stopInternalModification}
    '''
    self.__internalModification = True


  def stop( self ):
    '''
    Stop the timer if it is active.
    '''
    self.__timer.stop()

  
  def isActive( self ):
    '''
    Returns True if the timer is active, or False otherwise.
    '''
    return self.__timer.isActive()


#-------------------------------------------------------------------------------
class TimeredQLineEdit( QtGui.QLineEdit ):
  '''
  Create a L{QLineEdit<QtGui.QLineEdit>} instance that has an private attribute
  containing a L{QLineEditModificationTimer} associated to C{self}. Whenever
  the internal L{QLineEditModificationTimer} emits a C{SIGNAL( 
  'userModification' )} signal, this signal is also emited by the 
  L{TimeredQLineEdit} instance.
  '''
  def __init__( self, *args, **kwargs ):
    '''
    All non keyword parameters of the constructor are passed to
    L{QLineEdit<QtGui.QLineEdit>} constructor. An optional C{timerInterval} 
    keyword parameter can be given, it is passed to 
    L{QLineEditModificationTimer} constructor. At the time this class was 
    created, L{QLineEdit<qt.QLineEdit>} constructor did not accept keyword 
    parameters.
    '''
    timerInterval = kwargs.pop( 'timerInterval', None )
    if kwargs:
      QtGui.QLineEdit.__init__( self, *args, **kwargs )
    else:
      QtGui.QLineEdit.__init__( self, *args )
    self.__timer = QLineEditModificationTimer( self,
                                               timerInterval=timerInterval )
    self.connect( self.__timer, QtCore.SIGNAL( 'userModification' ),
                  QtCore.SIGNAL( 'userModification' ) )


  def stopInternalModification( self ):
    '''
    @see: L{QLineEditModificationTimer.stopInternalModification}
    '''
    self.__timer.stopInternalModification()


  def startInternalModification( self ):
    '''
    @see: L{QLineEditModificationTimer.startInternalModification}
    '''
    self.__timer.startInternalModification()

  def close( self ):
    self.__timer.close()
    super( TimeredQLineEdit, self ).close()


# -*- coding: utf-8 -*-

try:
    from traits.api import HasTraits
except:
    from enthought.traits.api import HasTraits
from soma.qt_gui.qt_backend import set_qt_backend, loadUi
from soma.factory import Factories
from soma.controller import Controller
from soma.application import Application
from soma.gui.widget_controller_creation import ControllerWidget


class WidgetFactories( Factories ):  
  def get_global_factory( self, key ):
    name = getattr( key, '__name__', None )
    #print '!get_global_factory!', key, name
    #if name:
      #import_feature( 'widget_factory.' + name )
    result = super( WidgetFactories, self ).get_global_factory( key )
    #print '!get_global_factory! ->', result, self._global_factories.keys()
    return result
    
  
  def create_widget( self, object, parent=None, live=False ):
    factory = self.get_factory( object )
    if factory is not None:
      return factory( object, parent=parent, live=live )
    if not isinstance( object, Controller ):
      controller = Application().get_controller( object )
      if controller is not None:
        return self.create_widget( controller, parent=parent, live=live )
    return None


def controller_widget_factory( controller, parent=None, live=False ):
 return ControllerWidget( controller, parent=parent, live=live )
 #trait_view = has_traits.trait_view()
  #trait_view.buttons = []
  #if live:
    #kwargs = { 'kind': 'live' }
  #else:
    #kwargs = {}
  #trait_ui = trait_view.ui( has_traits, **kwargs )
  #if parent is not None:
    #trait_ui.control.setParent( parent )
  #trait_ui.control._trait_ui = trait_ui
  #return trait_ui.control

WidgetFactories.register_global_factory( Controller, traits_widget_factory )


def create_widget_from_ui( ui, controller, parent=None, live=False ):
  set_qt_backend()
  widget = loadUi( ui )
  if parent is not None:
    widget.setParent( parent )
  if live:
    Application().gui.connect_widget( controller, widget )
  return widget




# -*- coding: utf-8 -*-
from soma.qt_gui.qt_backend import QtCore, QtGui
from soma.qt_gui.qt_backend.QtCore import QObject, SIGNAL
from soma.qt_gui.qt_backend.QtGui import QWidget, QVBoxLayout

from soma.application import Application
from soma.controller import trait_ids


class WidgetToControllerConnector( object ):
  @staticmethod
  def widget_ids( widget ):
    return [ klass.__name__ for klass in widget.__class__.__mro__ if issubclass( klass, QWidget ) ]
  
  default_factory = {}
  
  
  def __init__( self ):
    self.factory = {}
  
  def get_factory( self, trait_id, widget_id ):
    key = ( trait_id, widget_id )
    factory = self.factory.get( key )
    if factory is None:
      factory = self.default_factory.get( key )
    return factory
  
  
  def connect( self, controller, widget ):
    #print '!connect!', controller, widget
    for trait_name in controller.trait_names():
      #print '!connect! 2', trait_name
      subwidget = getattr( widget, trait_name, None )
      if isinstance( subwidget, QWidget ):
        #print '!connect! 3', subwidget
        for trait_id in trait_ids( controller.trait( trait_name ) ):
          for widget_id in self.widget_ids( subwidget ):
            #print '!connect! 4', trait_id, widget_id
            connector_factory = self.get_factory( trait_id, widget_id )
            if connector_factory is not None:
              connector = connector_factory( controller, trait_name, subwidget )
              #print '!connect! 5', connector
              break



class TraitWidgetConnector( QObject ):
  def __init__( self, controller, attribute_name, widget ):
    super( QObject, self ).__init__( widget )
    self.controller = controller
    self.attribute_name = attribute_name
    self._ignore_update = False
    self.connect_widget()
    self.controller.on_trait_change( self._update_widget, self.attribute_name )
    self.update_widget()
  
  def __del__( self ):
    self.disconnect_widget()
    self.controller.on_trait_change( self._update_widget, self.attribute_name, remove=True )

  def _update_widget( self ):
    if self._ignore_update == False:
      self._ignore_update = True
      try:
        self.update_widget()
      finally:
        self._ignore_update = False

class Connector_Unicode_QLineEdit( TraitWidgetConnector ):
  unicode_to_attribute = lambda x: x
  attribute_to_unicode = unicode
  
  def connect_widget( self ):
    self.connect( self.parent(), SIGNAL( 'textChanged( const QString & )' ), self.update_controller )
  
  def disconnect_widget( self ):
    try:
      self.disconnect( self.parent(), SIGNAL( 'textChanged( const QString & )' ), self.update_controller )
    except RuntimeError:
      pass
  
  def update_widget( self ):
    self.parent().setText( self.attribute_to_unicode( getattr( self.controller, self.attribute_name ) ) )

  def update_controller( self ):
    setattr( self.controller, self.attribute_name, self.unicode_to_attribute( self.parent().text() ) )
WidgetToControllerConnector.default_factory[ ( 'Unicode', 'QLineEdit' ) ] = Connector_Unicode_QLineEdit
WidgetToControllerConnector.default_factory[ ( 'String', 'QLineEdit' ) ] = Connector_Unicode_QLineEdit
WidgetToControllerConnector.default_factory[ ( 'Str', 'QLineEdit' ) ] = Connector_Unicode_QLineEdit


class Connector_Float_QLineEdit( Connector_Unicode_QLineEdit ):
  unicode_to_attribute = float
WidgetToControllerConnector.default_factory[ ( 'Float', 'QLineEdit' ) ] = Connector_Float_QLineEdit


class Connector_Int_QLineEdit( Connector_Unicode_QLineEdit ):
  unicode_to_attribute = int
WidgetToControllerConnector.default_factory[ ( 'Int', 'QLineEdit' ) ] = Connector_Int_QLineEdit


class Connector_Bool_QCheckBox( TraitWidgetConnector ):
  def connect_widget( self ):
    self.connect( self.parent(), SIGNAL( 'stateChanged ( int )' ), self.update_controller )
  
  def disconnect_widget( self ):
    TODO
    self.disconnect( self.parent(), SIGNAL( 'stateChanged ( int )' ), self.update_controller )
  
  def update_widget( self ):
    self.parent().setChecked( getattr( self.controller, self.attribute_name ) )

  def update_controller( self ):
    setattr( self.controller, self.attribute_name, self.parent().isChecked() )
WidgetToControllerConnector.default_factory[ ( 'Bool', 'QCheckBox' ) ] = Connector_Bool_QCheckBox


class Connector_ListFloat_QLineEdit( TraitWidgetConnector ):
  def __init__( self, controller, attribute_name, widget ):
    super( Connector_ListFloat_QLineEdit, self ).__init__( controller, attribute_name, widget )
    self.controller.on_trait_change( self._update_widget, attribute_name + '_items' )
  
  def connect_widget( self ):
    self.connect( self.parent(), SIGNAL( 'textChanged( const QString & )' ), self.update_controller )
    
  def disconnect_widget( self ):
    self.disconnect( self.parent(), SIGNAL( 'textChanged( const QString & )' ), self.update_controller )
    
  def update_widget( self ):
    self.parent().setText( ' '.join( ( str(i) for i in getattr( self.controller, self.attribute_name ) ) ) )

  def update_controller( self ):
    setattr( self.controller, self.attribute_name, [ float(i) for i in self.parent().text().split() ] )
WidgetToControllerConnector.default_factory[ ( 'List_Float', 'QLineEdit' ) ] = Connector_ListFloat_QLineEdit


class Connector_Float_QSlider( TraitWidgetConnector ):
  def connect_widget( self ):
    self.connect( self.parent(), SIGNAL('valueChanged( int )' ), self.update_controller )
    
  def disconnect_widget( self ):
    self.disconnect( self.parent(), SIGNAL( 'valueChanged( int )' ), self.update_controller )
    
  def update_widget( self ):
    self.parent().setValue( getattr( self.controller, self.attribute_name ) )

  def update_controller( self ):
    setattr( self.controller, self.attribute_name, self.parent().value() )
WidgetToControllerConnector.default_factory[ ( 'Float', 'QSlider' ) ] = Connector_Float_QSlider


class Connector_object_QFrame( TraitWidgetConnector ):
  def connect_widget( self ):
    self.layout = QVBoxLayout( self.parent() )
    self.subwidget = Application().gui.create_widget( getattr( self.controller, self.attribute_name ), parent=self.parent(), live=True )
    if self.subwidget is not None:
      self.layout.addWidget( self.subwidget )
  
  def disconnect_widget( self ):
    pass
  
  def update_widget( self ):
    pass

  def update_controller( self ):
    pass
WidgetToControllerConnector.default_factory[ ( 'Instance_object', 'QFrame' ) ] = Connector_object_QFrame



#! /usr/bin/env python
##########################################################################
# Soma-Base - Copyright (C) CEA, 2013
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
##########################################################################

"""This module enables to create automatic links between an object and a
traits based controller."""

try:
    from traits.api import HasTraits, Event
except ImportError:
    from enthought.traits.api import HasTraits, Event

from weakref import WeakKeyDictionary
from soma.utils.functiontools import partial
from soma.sorted_dictionary import SortedDictionary

from soma.controller.factory import Factories

global_compt_order=0


class ControllerFactories( Factories ):
    """Holds association between an object and its controller"""
    def __init__( self ):
        super( ControllerFactories, self ).__init__()
        self._controllers = WeakKeyDictionary()

    def get_controller( self, instance ):
        """Returns the Controller class associated to an object."""
        controller = self._controllers.get( instance )
        if controller is None:
            factory = self.get_factory( instance )
            if factory is not None:
                controller = factory( instance )
                self._controllers[ instance ] = controller
        return controller

ControllerFactories.register_global_factory( HasTraits, lambda instance: instance )



class MetaController( HasTraits.__metaclass__ ):
    """This metaclass allows for automatic registration of a Controller derived
    class for ControllerFactories (if the new class defines register_class_controller)
    and WidgetFactories (if the new class defines create_widget_from_ui)"""
    def __new__( mcs, name, bases, dictionary ):
        cls = super( MetaController, mcs ).__new__( mcs, name, bases, dictionary )

#        controlled_class = dictionary.get( 'register_class_controller' )
#        if controlled_class is not None:
#            ControllerFactories.register_global_factory( controlled_class, cls )
#
#        gui = dictionary.get('create_widget_from_ui')
#        if gui is not None:
#            from soma.gui.widget_factory import WidgetFactories, create_widget_from_ui
#            WidgetFactories.register_global_factory( cls, partial( create_widget_from_ui, gui ) )

        return cls


class Controller(HasTraits):
    """
    A Controller is a HasTraits that is connected to ControllerFactories and
    widgetFactories, it also provides some methods to inspect user defined traits
    and to raise an event if its traits have changed.
    """
    __metaclass__ = MetaController

    """
    This event is necessary because there is no event when a trait is removed
    with remove_trait and because it is sometimes better to send a single event
    when several traits changes are done (especially when GUI is updated on real
    time). This event have to be triggered explicitely to take into account
    changes due to call(s) to add_trait or remove_trait.
    """
    user_traits_changed = Event

    def __init__(self, *args, **kwargs):
        super(Controller, self).__init__(*args, **kwargs)
        self._user_traits = SortedDictionary()
        class_traits = self.class_traits()
        if class_traits:
          sorted_keys = [t[1]
              for t in sorted((getattr(trait, 'order', ''), name)
              for name, trait in class_traits.iteritems() if self.is_user_trait(trait))]
          for name in sorted_keys:
              self._user_traits[name] = class_traits[name]

    def user_traits(self):
        """
        Returns a dictionnary containing class traits and instance traits
        defined by user (i.e.  the traits that are not automatically
        defined by HasTraits or Controller).
        Returned values are sorted according to the "order" trait
        meta-attribute.
        """
        return self._user_traits

    def is_user_trait(self, trait):
        '''
        Test if a trait is a valid user trait (i.e. not an Event).
        '''
        return not isinstance(trait.handler, Event)

    def add_trait(self, name, *trait):   
        super(Controller, self).add_trait(name, *trait)
        trait_instance = self.trait(name)
        if self.is_user_trait(trait_instance):
            trait_instance.defaultvalue = trait_instance.default
            #self.get(name)
            self._user_traits[name] = trait_instance

    def remove_trait(self,name):
        super(Controller, self).remove_trait(name)
        # Remove name from _user_traits without error if it
        # is not present
        self._user_traits.pop(name,None)




# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
This module contains functions and classes related to sqlite databases.

@author: Yann Cointepas
@organization: U{NeuroSpin<http://www.neurospin.org>} and U{IFR 49<http://www.ifr49.org>}
@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''
from __future__ import absolute_import
__docformat__ = "epytext en"


import sys
import threading
import sqlite3

#------------------------------------------------------------------------------
class ThreadSafeSQLiteConnection( object ):
  '''
  Python wrapping of SQLite do not allow sharing of database connection between
  threads. This class allows to automatically create a connection for each
  thread.
  '''
  _currentId = 0
  _classLock = threading.RLock()
    
  def __init__( self, *args, **kwargs ):
    super( ThreadSafeSQLiteConnection, self ).__init__()
    self.__args = args
    self.__kwargs = kwargs
    self._instanceLock = threading.RLock()
    self.connections = {}
    self._classLock.acquire()
    try:
      self._id = ThreadSafeSQLiteConnection._currentId
      ThreadSafeSQLiteConnection._currentId += 1
    finally:
      self._classLock.release()

  def __del__( self ):
    if threading is None:
      # The interpretor is exiting and we cannot access threading
      # module. We cannot do anything.
      return
    if self.__args is not None:
      sqliteFile = self.__args[ 0 ]
      self.close()
      for thread in self.connections.keys():
        connection, connectionClosed = self.connections[ thread ]
        if connection is not None:
          currentThread = threading.currentThread().getName()
          print >> sys.stderr, 'WARNING: internal error: an sqlite connection on', repr( sqliteFile ), 'is opened for thread', thread, 'but the corresponding ThreadSafeSQLiteConnection instance (number ' + str( self._id ) + ') is being deleted in thread', currentThread + '. Method currentThreadCleanup() should have been called from', thread, 'to supress this warning.'
  
  def _getConnection( self ):
    if self.__args is None:
      raise RuntimeError( 'Attempt to access to a closed ThreadSafeSQLiteConnection' )
    currentThread = threading.currentThread().getName()
    #print '!ThreadSafeSQLiteConnection:' + currentThread + '! _getConnection( id =', self._id, ')', self.__args
    self._instanceLock.acquire()
    try:
      #currentThreadConnections = self.connections.setdefault( currentThread, {} )
      #print '!ThreadSafeSQLiteConnection:' + currentThread + '! currentThreadConnections =', currentThreadConnections
      connection, connectionClosed = self.connections.get( currentThread, ( None, True ) )
      if connectionClosed:
        if connection is not None:
          connection.close()
        connection = sqlite3.connect( *self.__args, **self.__kwargs )
        #print '!ThreadSafeSQLiteConnection:' + currentThread + '! opened', connection
        self.connections[ currentThread ] = ( connection, False )
    finally:
        self._instanceLock.release()
    return connection
  
  
  def currentThreadCleanup( self ):
    currentThread = threading.currentThread().getName()
    self._instanceLock.acquire()
    try:
      connection, connectionClosed = self.connections.pop( currentThread, ( None, True ) )
    finally:
      self._instanceLock.release()
    if connection is not None:
        connection.close()
  
  
  def close( self ):
    if self.__args is not None:
      self.closeSqliteConnections()
      self.__args = None
      self.__kwargs = None


  def closeSqliteConnections( self ):
    if self.__args is not None:
      self.currentThreadCleanup()
      currentThread = threading.currentThread().getName()
      self._instanceLock.acquire()
      try:
        for thread in self.connections.keys():
          connection, connectionClosed = self.connections[ thread ]
          self.connections[ thread ] = ( connection, True )
      finally:
        self._instanceLock.release()




# -*- coding: utf-8 -*-
# modified zipfile module in brainvisa project to switch between the regular
# python zipfile and a copy of the one from python 2.6 when version < 2.6

import sys, os
ver = sys.version_info[0] * 0x100 + sys.version_info[1]
if ver >= 0x0206:
  import imp
  f, pathname, description = imp.find_module( 'zipfile',
    [ os.path.dirname( os.__file__ ) ] )
  try:
    zipfile = imp.load_module( 'zipfile', f, pathname, description )
    globals().update( zipfile.__dict__ )
  finally:
    f.close()
    del f, pathname, description, imp
else:
  from zipfile import *
del zipfile, ver, sys, os




# -*- coding: utf-8 -*-

import re
from inspect import isfunction, isclass, ismethod
from weakref import WeakKeyDictionary

from soma.singleton import Singleton

class GlobalNaming( Singleton ):
  def __singleton_init__( self ):
    super( GlobalNaming, self ).__singleton_init__()
    self._global_name_re = re.compile( r'(([A-Za-z][A-Za-z0-9_.]*)\.([A-Za-z0-9_]+))(\(\))?((\.)([A-Za-z][A-Za-z0-9_.]+))?' )
    self._names = WeakKeyDictionary()


  def parse_global_name( self, global_name ):
    m = self._global_name_re.match( global_name )
    if m:
      # return module, name, args, attributes
      return m.group(2), m.group(3), ( () if m.group(4) else None ), m.group(7)
    raise ValueError( 'Invalid global name syntax: ' + global_name )


  def get_object( self, global_name ):
    module, name, args, attributes = self.parse_global_name( global_name )
    module = __import__( module, fromlist=[ name ], level=0 )
    try:
      value = getattr( module, name )
    except AttributeError, e:
      raise ImportError( str( e ) )
    if args is not None:
      value = value( *args )
    if attributes:
      for a in attributes.split( '.' ):
        value = getattr( value, a )
      self._names[ value ] = global_name
    return value

  
  def get_name( self, obj ):
    result = self._names.get( obj )
    if result is not None:
      return result
    if isfunction( obj ) or isclass( obj ):
      return obj.__module__ + '.' + obj.__name__
    elif ismethod( obj ):
      if obj.im_self is not None:
        return obj.im_class.__module__ + '.' + obj.im_class.__name__ + '().' + obj.im_func.__name__
    elif isinstance( obj, object ):
      return obj.__class__.__module__ + '.' + obj.__class__.__name__ + '()'
    raise ValueError( 'Cannot find global name for %s' % repr( obj ) )

    
def get_object( global_name ):
  return GlobalNaming().get_object( global_name )

  
def get_name( obj ):
  return GlobalNaming().get_name( obj )



# -*- coding: iso-8859-1 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
Universal unique identifier.

@author: Yann Cointepas
@organization: U{NeuroSpin<http://www.neurospin.org>} and U{IFR 49<http://www.ifr49.org>}
@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''
__docformat__ = "epytext en"

import struct, random, binascii
import types

#-------------------------------------------------------------------------------
class Uuid( object ):
  '''
  An Uuid instance is a universal unique identifier. It is a 128 bits
  random value.
  '''
  def __new__( cls, value=None ):
    if isinstance( value, Uuid ):
      return value
    return object.__new__( cls )
  
  
  def __init__( self, uuid=None ):
    '''
    Uuid constructor. If C{uuid} is ommited or C{None}, a new random 
    Uuid is created, if it is a string if must be 36 characters long and follow 
    the pattern C{XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX} where C{X} is an 
    hexadecimal digit (example: C{'ad2d8fb0-7831-50bc-2fb6-5df048304001'}). If 
    C{uuid} is a Uuid instance, no new instance is created, in this 
    case, C{Uuid( uuid )} returns C{uuid}.
    '''
    if isinstance( uuid, Uuid ): return
    if uuid is None:
      # Generate a new 128 bits uuid
      self.__uuid = struct.pack( 'QQ', random.randrange( 2**64-1 ),
                                 random.randrange( 2**64-1 ) )
    else:
      try:
        self.__uuid = binascii.unhexlify( uuid[0:8] + uuid[9:13] + \
                                          uuid[14:18] + uuid[19:23] +\
                                          uuid[24:36] )
      except:
        raise ValueError( "Invalid uuid string %s" % ( repr( uuid ), ) )

  def __getinitargs__( self ):
    return ( str( self ), )
  
  
  def __str__( self ):
    return binascii.hexlify( self.__uuid[0:4] ) + '-' + \
           binascii.hexlify( self.__uuid[4:6] ) + '-' + \
           binascii.hexlify( self.__uuid[6:8] ) + '-' + \
           binascii.hexlify( self.__uuid[8:10] ) + '-' + \
           binascii.hexlify( self.__uuid[10:16] )
  
  
  def __repr__( self ):
    return repr( str( self ) )
  
  
  def __hash__( self ):
    return hash( self.__uuid )
  
  
  def __eq__( self, other ):
    if isinstance( other, Uuid ):
      return self.__uuid == other.__uuid
    elif isinstance(other, types.StringTypes): # assume string-like object (str or unicode)
      return self.__uuid == Uuid( other ).__uuid
    else:
      return False

  def __ne__( self, other ):
    if isinstance( other, Uuid ):
      return self.__uuid != other.__uuid
    elif isinstance(other, types.StringTypes): # assume string-like object (str or unicode)
      return self.__uuid != Uuid( other ).__uuid
    else:
      return True



# -*- coding: utf-8 -*-
import sipconfig
from soma.qt_gui.qt_backend import QtGui, QtCore
from soma.controller import trait_ids, Controller
from soma.functiontools import partial, SomaPartial
from trait_to_widget import StrCreateWidget,FloatCreateWidget,IntCreateWidget,\
LongCreateWidget,BoolCreateWidget,EnumCreateWidget,StrEnumCreateWidget,\
FileCreateWidget,DirectoryCreateWidget
from soma.gui.icon_factory import IconFactory


class ControllerWidget( QtGui.QWidget ):
  # QPixmap instance cannot be build before a QApplication is being created.
  # (it simply exits the program). But for documentation with epydoc, all
  # modules are loaded without C{QApplication}. Therefore, the creation of
  # the following two static QPixmap instances has been put in __init__.

  _create_widget = {}

  def __init__( self, controller,
                parent=None, name=None, live=False, hide_labels=False ):

    super( ControllerWidget, self ).__init__( parent )
    if name:
      self.setObjectName(name)
    self.controller = controller
    self.live = live
    self.connected = False
    self.hide_labels = hide_labels
    self._grid_layout = QtGui.QGridLayout()
    self._grid_layout.setAlignment( QtCore.Qt.AlignTop )
    self._grid_layout.setSpacing( 3 )
    self._grid_layout.setContentsMargins( 5, 5, 5, 5 )
    self.setLayout(self._grid_layout )
    
    self.btn_expand = None
    self._widgets = {}
    self._create_widgets()
    if self.live:
      self.connect_controller()
    else:
     self.update_controller_widget()
    

 
  def update_controller( self ):
    """Update traits when GUI change"""
    for name, tuple in self._widgets.iteritems():
      trait, create_widget, attribute_widget, label_widget = tuple
      create_widget.update_controller( self, name, attribute_widget )
    
  
  def update_controller_widget( self ):
    """Update GUI when traits change"""
    for name, tuple in self._widgets.iteritems():
      trait, create_widget, attribute_widget, label_widget = tuple
      create_widget.update_controller_widget( self, name, attribute_widget, label_widget )
      
  
  ##def back_default_value(self,attribute_widget,name):  
      #print 'here back default value' 
      #print self.controller.trait(name)
      #print self.controller.trait(name).defaultvalue
      #setattr( self.controller, name, self.controller.trait(name).defaultvalue )
      #attribute_widget.btn_default_value.setVisible(False)
      
  
  
  def connect_controller( self ):
    """To connect traits and GUI, use update_controller and update controller_widget"""
    if not self.connected:  
      for name, tupl in self._widgets.iteritems():
        trait, create_widget, attribute_widget, label_widget = tupl
        create_widget.connect_controller( self, name, attribute_widget, label_widget )
      self.controller.on_trait_change( self.update_widgets, 'user_traits_changed' )
      self.update_controller_widget()
      self.connected = True


  def disconnect_controller( self ):
    """To disconnect traits and GUI"""
    if self.connected:
      self.controller.on_trait_change( self.update_widgets, 'user_traits_changed', remove=True )
      for name, tupl in self._widgets.iteritems():
        trait, create_widget, attribute_widget, label_widget = tupl
        create_widget.disconnect_controller( self, name, attribute_widget, label_widget )
      self.connected = False

  @classmethod
  def find_create_widget_from_trait( cls, trait ):
    """Find widget corresponding to the trait, call the class create_create widget corresponding"""
    create_widget = None
    for trait_id in trait_ids( trait ):
      for create_widget in cls._create_widget.get( trait_id, [] ):
        if create_widget.is_valid_trait( trait ):
          break
      else:
        create_widget = None
      if create_widget is not None:
        break
    return create_widget


  def _create_widgets( self ):
    # print '!_create_widgets!', self.controller.user_traits().keys()
    for name, trait in self.controller.user_traits().iteritems():
      create_widget = self._widgets.get( name )
      if create_widget is None:
        create_widget = self.find_create_widget_from_trait( trait )
        if not create_widget:
          continue
        attribute_widget, label_widget = create_widget.create_widget( self, name, trait, getattr( self.controller, name ) )
        if trait.desc:
          tooltip = '<b>' + name + ':</b> ' + trait.desc
        else:
          tooltip = None
        if label_widget is None:
          self._grid_layout.addWidget( attribute_widget, self._grid_layout.rowCount(), 0, 1, 2 )
          if tooltip:
            attribute_widget.setToolTip( tooltip )
        elif isinstance( label_widget, tuple ):
          row = self._grid_layout.rowCount()
          if label_widget[0] and not self.hide_labels:
            self._grid_layout.addWidget( label_widget[0], row, 0 )
            if tooltip:
              label_widget[0].setToolTip( tooltip )
          self._grid_layout.addWidget( label_widget[1], row, 1 )
          if tooltip:
            label_widget[1].setToolTip( tooltip )
          self._grid_layout.addWidget( attribute_widget, self._grid_layout.rowCount(), 0, 1, 2 )
        else:
          row = self._grid_layout.rowCount()
          if self.hide_labels:
            self._grid_layout.addWidget( attribute_widget, row, 0, 1, 2 )
            if tooltip:
              attribute_widget.setToolTip( tooltip )
          else: 
            self._grid_layout.addWidget( label_widget, row, 0 )
            self._grid_layout.addWidget( attribute_widget, row, 1 )
            #btn_default_value = QtGui.QPushButton()
            #btn_default_value.setIcon( QtGui.QIcon( IconFactory()._valueModified) )
            #btn_default_value.setVisible(False)
            #attribute_widget.btn_default_value = btn_default_value  
            #self._grid_layout.addWidget( btn_default_value,row,2 )
            if tooltip: 
              label_widget.setToolTip( tooltip )
              attribute_widget.setToolTip( tooltip )
        self._widgets[ name ] = ( trait, create_widget, attribute_widget, label_widget ) 
      else:
        trait, create_widget, attribute_widget, label_widget = create_widget
      if getattr( trait, 'hidden', False ):
        attribute_widget.hide()
        if label_widget:
          if isinstance( label_widget, tuple ):
            l = label_widget
          else:
            l = [ label_widget ]
          for i in l:
            i.hide()
      else:
        attribute_widget.show()
        if label_widget:
          if isinstance( label_widget, tuple ):
            l = label_widget
          else:
            l = [ label_widget ]
          for i in l:
            i.show()


  def update_widgets( self ):
    user_traits = self.controller.user_traits()
    was_connected = self.connected
    if was_connected:
      self.disconnect_controller()
    for name in self._widgets.keys():
      trait, create_widget, attribute_widget, label_widget = self._widgets[ name ]
      if user_traits.get( name ) != trait:
        attribute_widget.close()
        attribute_widget.deleteLater()
        if isinstance( label_widget, tuple ):
          if label_widget[ 0 ]:
            label_widget[ 0 ].close()
            label_widget[ 0 ].deleteLater()
          if label_widget[ 1 ]:
            label_widget[ 1 ].close()
            label_widget[ 1 ].deleteLater()
        elif label_widget:
          label_widget.close()
          label_widget.deleteLater()
        del self._widgets[ name ]
    self._create_widgets()
    if was_connected:
      self.connect_controller()
    self.updateGeometry()


#-------------------------------------------------------------------------------
class ListCreateWidget( object ):
  class ListController( Controller ):
    pass

  @staticmethod
  def is_valid_trait( trait ):
    item_trait = trait.inner_traits[ 0 ]
    create_widget = ControllerWidget.find_create_widget_from_trait( item_trait )
    return create_widget is not None
  
  @classmethod
  def create_widget( cls, parent, name, trait, value ):   
    item_trait = trait.inner_traits[ 0 ]
    #print '!ListCreateWidget!', name, 'List( %s )' % trait_ids( item_trait )
    list_controller = cls.ListController()
    for i in xrange( len( value ) ):
      list_controller.add_trait( str( i ), item_trait )
      trait = list_controller.trait( str( i ) )
      trait.order = i
      setattr( list_controller, str( i ), value[ i ] )
    result = ControllerCreateWidget.create_widget( parent, name, None, list_controller )
    attribute_widget = result[ 0 ]
    attribute_widget.controller_widget.connect_controller()
    attribute_widget.item_trait = item_trait
    attribute_widget.list_controller = list_controller
    attribute_widget.connected = False
    return result

  @staticmethod
  def update_controller( controller_widget, name, attribute_widget ):
    items = [ getattr( attribute_widget.list_controller, str( i ) ) \
      for i in xrange( len( attribute_widget.list_controller.user_traits() ) ) ]
    #print '!update_controller!', name, len( items ), items
    setattr( controller_widget.controller, name, items )
  
  
  @classmethod
  def update_controller_widget( cls,controller_widget, name, attribute_widget, label_widget ):
    was_connected = attribute_widget.connected
    cls.disconnect_controller( controller_widget, name, attribute_widget, label_widget )
    attribute_widget.controller_widget.disconnect_controller()
    items = getattr( controller_widget.controller, name )
    len_widget = len( attribute_widget.list_controller.user_traits() )
    #print '!update_controller_widget!', name, len_widget, items
    user_traits_changed = False
    if len( items ) < len_widget:
      for i in xrange( len( items ), len_widget ):
        attribute_widget.list_controller.remove_trait( str( i ) )
      user_traits_changed = True
    elif len( items ) > len_widget:
      for i in xrange( len_widget, len( items ) ):
        attribute_widget.list_controller.add_trait( str( i ), attribute_widget.item_trait )
        trait = attribute_widget.list_controller.trait( str( i ) )
        trait.order = i
      user_traits_changed = True
    for i in xrange( len( items ) ):
      setattr( attribute_widget.list_controller, str( i ), items[ i ] )
    #print '!update_controller_widget! done', name
    attribute_widget.controller_widget.connect_controller()
    if user_traits_changed:
      attribute_widget.list_controller.user_traits_changed = True
    if was_connected:
      cls.connect_controller( controller_widget, name, attribute_widget, label_widget )
  


  @classmethod
  def connect_controller( cls, controller_widget, name, attribute_widget, label_widget ):
    if not attribute_widget.connected:
      def list_controller_hook( obj, key, old, new ):
        #print '!list_controller_hook!', ( obj, key, old, new )
        items = getattr( controller_widget.controller, name )
        items[ int( key ) ] = new
      for n in attribute_widget.list_controller.user_traits():
        attribute_widget.list_controller.on_trait_change( list_controller_hook, n )
      controller_hook = SomaPartial( cls.update_controller_widget, controller_widget, name, attribute_widget, label_widget )
      controller_widget.controller.on_trait_change( controller_hook, name + '[]' )
      attribute_widget._controller_connections = ( list_controller_hook, controller_hook)
      attribute_widget.connected = True
  
  @staticmethod
  def disconnect_controller( controller_widget, name, attribute_widget, label_widget ):
    if attribute_widget.connected:
      list_controller_hook, controller_hook = attribute_widget._controller_connections
      controller_widget.controller.on_trait_change( controller_hook, name + '[]', remove=True )
      for n in attribute_widget.list_controller.user_traits():
        attribute_widget.list_controller.on_trait_change( list_controller_hook, n, remove=True )
      del attribute_widget._controller_connections
      attribute_widget.connected = False
 


#-------------------------------------------------------------------------------
class ControllerCreateWidget( object ):
  """Class for instance trait"""    
  @staticmethod
  def is_valid_trait( trait ):
    return True
  
  @classmethod
  def create_widget( cls, parent, name, trait, sub_controller ):
    label = getattr( trait, 'label', None )
    if not label:
      label = name
    if label is not None:
      label_widget = QtGui.QLabel( label, parent )
    else:
      label_widget = None

    btn_expand = QtGui.QPushButton( parent )
    btn_expand.setIcon( QtGui.QIcon( IconFactory()._imageExpand ) )
    
    scroll_area = QtGui.QScrollArea( parent=parent )
    scroll_area.setWidgetResizable( True )
    #scroll_area.setSizePolicy( QtGui.QSizePolicy( QtGui.QSizePolicy.Expanding,
                                                  #QtGui.QSizePolicy.Expanding ) )                                           
    sub_controller_widget = ControllerWidget( sub_controller, 
      parent=scroll_area )
    scroll_area.setWidget( sub_controller_widget )
    #sub_controller_widget.setSizePolicy( QtGui.QSizePolicy( QtGui.QSizePolicy.Expanding,
                                                  #QtGui.QSizePolicy.MinimumExpanding ) )
    scroll_area.setFrameShape( QtGui.QFrame.StyledPanel )
    expand_hook = partial( cls.expand_collapse, scroll_area )
    btn_expand.connect( btn_expand, QtCore.SIGNAL( 'clicked()' ), 
                        expand_hook )
    attribute_widget = scroll_area
    attribute_widget.btn_expand = btn_expand
    attribute_widget.label_widget = label_widget
    attribute_widget.expand_hook = expand_hook
    scroll_area.hide()
    attribute_widget.controller_widget = sub_controller_widget
    return ( attribute_widget, ( label_widget, btn_expand ) )

  @staticmethod
  def update_controller( controller_widget, name, attribute_widget ): 
    attribute_widget.controller_widget.update_controller()
  
  @staticmethod
  def update_controller_widget( controller_widget, name, attribute_widget, label_widget ):
    attribute_widget.controller_widget.update_controller_widget()
  
  @staticmethod
  def connect_controller( controller_widget, name, attribute_widget, label_widget ):
    attribute_widget.controller_widget.connect_controller()

  @staticmethod
  def disconnect_controller( controller_widget, name, attribute_widget, label_widget ):
    attribute_widget.controller_widget.disconnect_controller()
  
  @staticmethod
  def expand_collapse( attribute_widget ):
    if attribute_widget.isVisible():
      attribute_widget.hide()
      attribute_widget.btn_expand.setIcon( QtGui.QIcon( IconFactory()._imageExpand ) )
    else:
      attribute_widget.show()
      attribute_widget.btn_expand.setIcon( QtGui.QIcon( IconFactory()._imageCollapse ) )




ControllerWidget._create_widget[ 'Str' ] = [ StrCreateWidget ]
ControllerWidget._create_widget[ 'Unicode' ] = [ StrCreateWidget ]
ControllerWidget._create_widget[ 'Float' ] = [ FloatCreateWidget ]
ControllerWidget._create_widget[ 'Int' ] = [ IntCreateWidget ]

ControllerWidget._create_widget[ 'Long' ] = [ LongCreateWidget ]
ControllerWidget._create_widget[ 'Bool' ] = [ BoolCreateWidget ]

ControllerWidget._create_widget[ 'Enum' ] = [ EnumCreateWidget ]
ControllerWidget._create_widget[ 'Str' ].insert( 0, StrEnumCreateWidget )
ControllerWidget._create_widget[ 'Unicode' ].insert( 0, StrEnumCreateWidget )
ControllerWidget._create_widget[ 'File' ] = [ FileCreateWidget ]
ControllerWidget._create_widget[ 'Directory' ] = [ DirectoryCreateWidget ]
ControllerWidget._create_widget[ 'Instance_Controller' ] = [ ControllerCreateWidget ]
ControllerWidget._create_widget[ 'Any' ] = [ StrCreateWidget ]

ControllerWidget._create_widget[ 'List_Str' ] = [ ListCreateWidget ]
ControllerWidget._create_widget[ 'List_Float' ] = [ ListCreateWidget ]
ControllerWidget._create_widget[ 'List_Int' ] = [ ListCreateWidget ]
ControllerWidget._create_widget[ 'List_Long' ] = [ ListCreateWidget ]
ControllerWidget._create_widget[ 'List_Bool' ] = [ ListCreateWidget ]
ControllerWidget._create_widget[ 'List_Enum' ] = [ ListCreateWidget ]
ControllerWidget._create_widget[ 'List_Directory' ] = [ ListCreateWidget ]
ControllerWidget._create_widget[ 'List_File' ] = [ ListCreateWidget ]
ControllerWidget._create_widget[ 'List_Any' ] = [ ListCreateWidget ]


#-------------------------------------------------------------------------------
#if __name__ == '__main__':
  #import sys
  #try:
    #from traits.api import Str, Unicode, Float, Int, Long, Bool, Enum, Directory, File, Instance, List
  #except:
    #from enthought.traits.api import Str, Unicode, Float, Int, Long, Bool, Enum, Directory, File, Instance, List
  #from soma.controller import Controller
  
  #app = QtGui.QApplication( sys.argv )
  
  #class InstanceTraits( Controller ):
    #def __init__( self ):
      #self.add_trait( 'str', Str( desc='This is a string', order=1, viewer='viewers.image.ImageViewer' ) )
      #self.add_trait( 'str2', Str( desc='This is a String', order=2 ) )
      #self.add_trait( 'unicode', Str( desc='This is a unicode', order=2 ) )
      #self.add_trait( 'float', Float( desc='This is a float', order=3 ) )
      #self.add_trait( 'int', Int( desc='This is an int', order=4 ) )
      #self.add_trait( 'long', Long( desc='This is a long', order=5 ) )
      #self.add_trait( 'bool', Bool( desc='This is a boolean.', order=6 ) )
      #self.add_trait( 'enum', Enum( 'first value', 'second value', 'third value', desc='This is an enum.', order=7 ) )
      #self.add_trait( 'str_enum', Str( desc='This is a StrEnum', values=( 'first value', 'second value', 'third value' ), order=8 ) )
      #self.add_trait( 'directory', Directory( desc='This is a directory.', order=9 ) )
      #self.add_trait( 'file', File( desc='This is a file.', order=10 ) )

      #self.add_trait( 'list_str', List( Str, desc='This is a string list', order=11 ) )
      #self.add_trait( 'list_unicode', List( Unicode, desc='This is a unicode list', order=12 ) )
      #self.add_trait( 'list_float', List( Float, desc='This is a float list', order=13 ) )
      #self.add_trait( 'list_int', List( Int, desc='This is an int list', order=14 ) )
      #self.add_trait( 'list_long', List( Long, desc='This is a long list', order=15 ) )
      #self.add_trait( 'list_bool', List( Bool, desc='This is a boolean list', order=16 ) )
      #self.add_trait( 'list_enum', List( Enum( 'first value', 'second value', 'third value' ), desc='This is an enum list', order=17 ) )
      #self.add_trait( 'list_str_enum', List( Str( values=( 'first value', 'second value', 'third value' ) ), desc='This is a StrEnum list', order=18 ) )
      #self.add_trait( 'list_directory', List( Directory, desc='This is a directory list', order=19 ) )
      #self.add_trait( 'list_file', List( File, desc='This is a file list', order=20 ) )



  #class WidgetOnGui( Controller ):
    ##lists_size = Int( 0, order=0, desc='Resize all the lists' )
    ##connected = Bool( default_value=True, order=1, desc='Disconnect one of the two widgets' )
    ##traits_on_class = Instance( ClassTraits, desc='All traits on this attribute are defined on class', order=2 )
    ##traits_on_instance0 = Instance( InstanceTraits, desc='All traits on this attribute are defined on class', order=2 )
    #traits_on_instance = Instance( SimpMorpho, desc='All traits on this attribute are defined on instance', order=3 )
    
    
    #def __init__( self ):
      #super( WidgetOnGui, self ).__init__()
      ##self.traits_on_class = ClassTraits()
      ##self.traits_on_instance0 = InstanceTraits()
      #self.traits_on_instance = SimpMorpho()
      #self.dummy = 'I am hidden'


    #def on_run(self):
      #print 'IN THE FUNCTION RUN'
      #self.traits_on_instance()
      
      
      
    

  
  #def resize( main, old, new ):
    ##for o in ( main.traits_on_class, main.traits_on_instance ):
    #for o in main.traits_on_instance :
      #for a in ( 'list_str', 'list_unicode', 'list_float', 'list_int', 'list_long', 'list_bool', 'list_enum', 'list_str_enum', 'list_directory', 'list_file' ):
            #list = getattr( o, a )
        #len_list = len( list )
        #if main.lists_size > len_list:
          ##print '!resize! + ', a, len_list, main.lists_size, list + [ o.trait( a ).inner_traits[0].default ] * ( main.lists_size - len_list )
          #setattr( o, a, list + [ o.trait( a ).inner_traits[0].default ] * ( main.lists_size - len_list ) )
          ##print '!resize! done'
        #elif main.lists_size < len_list:
          ##print '!resize! - ', a, len_list, main.lists_size
          #setattr( o, a, list[ : main.lists_size ] )
          ##print '!resize! done'
  
  #def f( connected ):
    #print '!f!', connected
    #if connected:
      #w1.connect_controller()
    #else:
      #w1.disconnect_controller()
  #o.on_trait_change( f, 'connected' )
  

  #w2 = ControllerWidget( o, live=True )
  #w2.show()
  
  #def f(object,name,old,new ):
    #print 'in f'  
    #print ( object,name,old,new )
  
  #o.traits_on_instance.on_trait_change( f )
  #o.on_trait_change( resize, 'lists_size' )
  #app.exec_()


  #w1.on_trait_change(update,'t1mri')
  
  #def update(object,name,old,new):
      #o.traits_on_instance._t1mri_changed()  



# -*- coding: utf-8 -*-
from soma.qt_gui.qt_backend import QtGui, QtCore
import os

ICON_PATH=os.path.join(os.path.dirname( __file__ ),'icons')


class IconFactory():
    """Class for the differents icon of the GUI"""  
    
    # QPixmap instance cannot be build before a QApplication is being created.
    # (it simply exits the program). But for documentation with epydoc, all
    # modules are loaded without C{QApplication}. Therefore, the creation of
    # the following two static QPixmap instances has been put in __init__.

    _imageExpand = None
    _imageCollapse = None
    _imageViewer=None
    _imageBrowseInput=None
    _imageBrowseOutput=None  
    _imageBrowseDir=None
    _valueModified=None
    
    def __init__(self):

        if IconFactory._imageExpand is None:
            IconFactory._imageExpand = QtGui.QPixmap()
            IconFactory._imageExpand.loadFromData( \
                "\x89\x50\x4e\x47\x0d\x0a\x1a\x0a\x00\x00\x00\x0d" \
                "\x49\x48\x44\x52\x00\x00\x00\x09\x00\x00\x00\x0b" \
                "\x08\x06\x00\x00\x00\xad\x59\xa7\x1b\x00\x00\x00" \
                "\x4f\x49\x44\x41\x54\x18\x95\x63\x7c\xf6\xec\x19" \
                "\x03\x21\xc0\x04\x63\x48\x49\x49\xfd\x47\x96\x40" \
                "\xe1\x43\x4d\xfa\x8f\x0b\x3f\x7b\xf6\x0c\x62\xd2" \
                "\xb3\x67\xcf\x18\xb1\x59\x03\x13\x67\x42\x17\xc0" \
                "\xc6\x67\xc2\x26\x81\xae\x81\x91\x24\xdf\xe1\x03" \
                "\x2c\x30\xc6\xdd\xbb\x77\xff\xa3\x4b\x2a\x2b\x2b" \
                "\xa3\x3a\x1c\x1f\x20\xca\x4d\x00\x23\x1d\x2b\x53" \
                "\x5e\xdc\x34\x20\x00\x00\x00\x00\x49\x45\x4e\x44" \
                    "\xae\x42\x60\x82"
                )
        if IconFactory._imageCollapse is None:
            IconFactory._imageCollapse = QtGui.QPixmap()
            IconFactory._imageCollapse.loadFromData( \
                "\x89\x50\x4e\x47\x0d\x0a\x1a\x0a\x00\x00\x00\x0d" \
                "\x49\x48\x44\x52\x00\x00\x00\x09\x00\x00\x00\x0b" \
                "\x08\x06\x00\x00\x00\xad\x59\xa7\x1b\x00\x00\x00" \
                "\x52\x49\x44\x41\x54\x18\x95\x8d\x90\xbb\x0d\x00" \
                "\x31\x08\x43\x0d\xba\x29\xdc\xb2\xff\x48\xb4\x5e" \
                "\x23\x57\xe5\x14\x91\x9c\x02\x15\x9f\x27\x1b\x30" \
                "\x49\xb8\xc5\x33\x93\xcc\x1c\x75\x18\x11\x06\x00" \
                "\x7e\x95\x01\x60\x1d\xbb\x96\xd2\x06\x91\xdc\x76" \
                "\xf3\x13\x50\x41\xaf\xc0\xa9\xf6\x3f\x8b\xb5\xff" \
                "\x5d\x47\x72\x48\xb2\x15\x98\x75\xeb\x05\x2f\xc0" \
                "\x1f\x1f\xf8\x34\x49\xac\xa1\x00\x00\x00\x00\x49" \
                "\x45\x4e\x44\xae\x42\x60\x82"
                )
		
	if IconFactory._imageBrowseDir is None:
            IconFactory._imageBrowseDir = QtGui.QPixmap(os.path.join(ICON_PATH,'browse.png'))    
	      
        if IconFactory._imageBrowseInput is None:
            IconFactory._imageBrowseInput = QtGui.QPixmap(os.path.join(ICON_PATH,'database_read.png'))   


        if IconFactory._imageBrowseOutput is None:
            IconFactory._imageBrowseOutput = QtGui.QPixmap(os.path.join(ICON_PATH,'database_write.png'))      


        if IconFactory._imageViewer is None:
            IconFactory._imageViewer = QtGui.QPixmap(os.path.join(ICON_PATH,'eye.png'))
            
            
        if IconFactory._valueModified is None:
            IconFactory._valueModified = QtGui.QPixmap(os.path.join(ICON_PATH,'icons/modified.png'))





# -*- coding: utf-8 -*-
from soma.qt_gui.qt_backend import QtCore, QtGui, getOpenFileName, \
    getExistingDirectory
from soma.qt4gui.api import TimeredQLineEdit

class FileSelectionWidget(QtGui.QWidget):
    ''' A widget with a label and file name field, and a button to select the
    file name.

    Parameters
    ----------
    selection_type: str
        select between file ('File') or directory ('Directory') mode
    label: str
        label displayed on the left
    label_h: int
        horizontal size of the label
    label_v: int
        vertical size of the label
    '''
    def __init__(self, selection_type, label, label_h=120, label_v=40):
        super(FileSelectionWidget, self).__init__()
        #Add a selection file in the constructor
        self.label = QtGui.QLabel(label)
        self.label.setFixedSize(label_h, label_v)
        self.button = QtGui.QPushButton('...')
        self.button.setFixedSize(30, 30)
        self.lay = QtGui.QHBoxLayout()
        self.selection_type = selection_type
        self.fname = None
        self.lineedit = TimeredQLineEdit()
        self.button.clicked.connect(self._on_button)
        self.connect(self.lineedit,
            QtCore.SIGNAL('userModification'), self._on_lineedit)
        self.lay.addWidget(self.label)
        self.lay.addWidget(self.lineedit)
        self.lay.addWidget(self.button)
        self.setLayout(self.lay)


    def _on_button(self):
        if self.selection_type == 'File':
            self.fname = getOpenFileName(self, 'Select file', '', '')
        elif self.selection_type == 'Directory':
            self.fname = getExistingDirectory( self, 'Select Directory', '',
                '')

        #check if something if fname
        if self.fname != '':
            self.lineedit.setText(self.fname)


    def _on_lineedit(self, text):
        self.fname = text
        self.emit(QtCore.SIGNAL("editChanged(const QString & )"), text)



# -*- coding: utf-8 -*-

#  This software and supporting documentation are distributed by
#      Institut Federatif de Recherche 49
#      CEA/NeuroSpin, Batiment 145,
#      91191 Gif-sur-Yvette cedex
#      France
#
# This software is governed by the CeCILL-B license under
# French law and abiding by the rules of distribution of free software.
# You can  use, modify and/or redistribute the software under the 
# terms of the CeCILL-B license as circulated by CEA, CNRS
# and INRIA at the following URL "http://www.cecill.info". 
# 
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL-B license and that you accept its terms.

'''
@author: Yann Cointepas
@organization: U{NeuroSpin<http://www.neurospin.org>} and U{IFR 49<http://www.ifr49.org>}
@license: U{CeCILL version 2<http://www.cecill.info/licences/Licence_CeCILL_V2-en.html>}
'''

from __future__ import absolute_import
__docformat__ = "restructuredtext en"


import os, sys, platform, traceback
from os.path import dirname

os.environ['ETS_TOOLKIT'] = 'qt4'
try:
  from enthought.traits.api import ReadOnly, Directory, ListStr, Instance
except ImportError:
  from traits.api import ReadOnly, Directory, ListStr, Instance

from soma.singleton import Singleton
from soma.controller import Controller, ControllerFactories

#-------------------------------------------------------------------------------
class Application( Singleton, Controller ):
  '''Any program using soma should create an Application instance to manage
  its configuration and to store any kind of value that have to be global to
  the program.'''
  name = ReadOnly( desc='Name of the application' )
  version = ReadOnly()
  
  plugin_modules = ListStr(
    desc='List of Python module to load after application configuration' )

  def __singleton_init__( self, name, version=None, *args, **kwargs ):
    '''Replaces __init__ in Singleton.'''
    super( Application, self ).__singleton_init__( *args, **kwargs )

    # Warning : Traits bug
    # Using the trait Directory() might instanciate a QApplication (seems to depend on the
    # traits release). If it is declared in the class, the QApplication is instanciated at
    # module importation which prevent to customize QApplication.
    self.add_trait( 'install_directory', Directory( 
      desc='Base directory where the application is installed' ) )
    self.add_trait( 'user_directory', Directory( 
      desc='Base directory where user specific information can be find' ) )
    self.add_trait( 'application_directory', Directory(
      desc='Base directory where application specifc information can be find' ) )
    self.add_trait( 'site_directory',  Directory( 
      desc='Base directory where site specifc information can be find' ) )
    self._controller_factories = None

    self.name = name
    self.version = version
    self.loaded_plugin_modules = {}


  def initialize( self ):
    '''This method must be called once to setup the application.'''
    self.install_directory = dirname( dirname( dirname( __file__ ) ) )
    homedir = os.getenv( 'HOME' )
    if not homedir:
      homedir = ''
      if platform.system() == 'Windows':
        homedir = os.getenv( 'USERPROFILE' )
        if not homedir:
          homedir = os.getenv( 'HOMEPATH' )
          if not homedir:
            homedir = '\\'
          drive = os.getenv( 'HOMEDRIVE' )
          if not drive:
            drive = os.getenv( 'SystemDrive' )
            if not drive:
              drive = os.getenv( 'SystemRoot' )
              if not drive:
                drive = os.getenv( 'windir' )
              if drive and len( drive ) >= 2:
                drive = drive[ :2 ]
              else:
                drive = ''
          homedir = drive + homedir
    if homedir and os.path.exists( homedir ):
      self.user_directory = homedir
    
    # Load early plugin modules
    for plugin_module in self.plugin_modules:
      module = self.load_plugin_module( plugin_module )
      if module is not None:
        self.loaded_plugin_modules[ plugin_module ] = module
        init = getattr( module, 'call_before_application_initialization', None )
        if init is not None:
          init( self )
        
    appdir = os.path.normpath( os.path.dirname( os.path.dirname( sys.argv[0] ) ) )
    if os.path.exists( appdir ):
      self.application_directory = appdir

    sitedir = os.path.join( '/etc', self.name )
    if os.path.exists( sitedir ):
      self.site_directory = sitedir

    # Load plugin modules
    for plugin_module in self.plugin_modules:
      module = self.loaded_plugin_modules.get( plugin_module )
      if module is None:
        module = self.load_plugin_module( plugin_module )
      if module is not None:
        self.loaded_plugin_modules[ plugin_module ] = module
        init = getattr( module, 'call_after_application_initialization', None )
        if init is not None:
          init( self )


  def get_controller( self, something ):
    '''This method must be used to get a controller for any object.'''
    if self._controller_factories is None:
      self._controller_factories = ControllerFactories()
    return self._controller_factories.get_controller( something )
  
  
  @staticmethod
  def load_plugin_module( plugin_module ):
    '''This method loads a plugin module. It imports the module without raising
    an axception if it fails.'''
    try:
      __import__( plugin_module, level=0 )
      return sys.modules[ plugin_module ]
    except:
      # Python 2.6 hack : print_last may fail here (maybe due to threads management)
      # traceback.print_last()
      exceptionType, exceptionValue, exceptionTraceback = sys.exc_info()
      traceback.print_exception(exceptionType, exceptionValue, exceptionTraceback)
    return None
    